=== modified file 'bzrlib/_static_tuple_c.c'
--- bzrlib/_static_tuple_c.c	2009-10-26 15:59:54 +0000
+++ bzrlib/_static_tuple_c.c	2009-10-27 09:25:31 +0000
@@ -184,6 +184,7 @@
 StaticTuple_FromSequence(PyObject *sequence)
 {
     StaticTuple *new;
+    PyObject *as_tuple = NULL;
     PyObject *item;
     Py_ssize_t i, size;
 
@@ -192,15 +193,17 @@
         return (StaticTuple *)sequence;
     }
     if (!PySequence_Check(sequence)) {
-        PyErr_Format(PyExc_TypeError, "Type %s is not a sequence type",
-                     Py_TYPE(sequence)->tp_name);
-        return NULL;
+        as_tuple = PySequence_Tuple(sequence);
+        sequence = as_tuple;
     }
     size = PySequence_Size(sequence);
-    if (size == -1)
+    if (size == -1) {
+        Py_XDECREF(as_tuple);
         return NULL;
+    }
     new = StaticTuple_New(size);
     if (new == NULL) {
+        Py_XDECREF(as_tuple);
         return NULL;
     }
     for (i = 0; i < size; ++i) {
@@ -209,10 +212,12 @@
         item = PySequence_GetItem(sequence, i);
         if (item == NULL) {
             Py_DECREF(new);
+            Py_XDECREF(as_tuple);
             return NULL;
         }
         StaticTuple_SET_ITEM(new, i, item);
     }
+    Py_XDECREF(as_tuple);
     return (StaticTuple *)new;
 }
 

=== modified file 'bzrlib/tests/test__static_tuple.py'
--- bzrlib/tests/test__static_tuple.py	2009-10-21 17:54:33 +0000
+++ bzrlib/tests/test__static_tuple.py	2009-10-27 09:25:31 +0000
@@ -578,6 +578,11 @@
         self.assertRaises(TypeError,
                           self.module.StaticTuple.from_sequence, foo='a')
 
+    def test_from_sequence_iterable(self):
+        st = self.module.StaticTuple.from_sequence(iter(('foo', 'bar')))
+        self.assertIsInstance(st, self.module.StaticTuple)
+        self.assertEqual(('foo', 'bar'), st)
+
     def test_pickle(self):
         st = self.module.StaticTuple('foo', 'bar')
         pickled = cPickle.dumps(st)

